/***************************************************************************
 *cr
 *cr            (C) Copyright 2010 The Board of Trustees of the
 *cr                        University of Illinois
 *cr                         All Rights Reserved
 *cr
 ***************************************************************************/

/* I/O routines for reading and writing matrices in column-major
 * layout
 */

#include <fstream>
#include <iostream>
#include <vector>

char *readFile(const char *fileName) {
  std::fstream f(fileName, std::fstream::in);
  if (!f.good()) {
    std::cerr << "Error Reading File!!" << std::endl;
    return NULL;
  }

  f.seekg(0, std::ios::end);
  int length = f.tellg();
  f.seekg(0, std::ios::beg);

  char *buffer;

  if (length > 0) {
    buffer = new char[length];
    f.read(buffer, length);
    buffer[length - 1] = 0;
  } else {
    buffer = new char;
    buffer[0] = 0;
  }

  f.close();

  return buffer;
}

bool readColMajorMatrixFile(const char *fn, int &nr_row, int &nr_col,
                            std::vector<float> &v) {
  std::cerr << "Opening file:" << fn << std::endl;
  std::fstream f(fn, std::fstream::in);
  if (!f.good()) {
    return false;
  }

  // Read # of rows and cols
  f >> nr_row;
  f >> nr_col;

  float data;
  std::cerr << "Matrix dimension: " << nr_row << "x" << nr_col << std::endl;
  while (f.good()) {
    f >> data;
    v.push_back(data);
  }
  v.pop_back(); // remove the duplicated last element
  return true;
}

bool writeColMajorMatrixFile(const char *fn, int nr_row, int nr_col,
                             std::vector<float> &v) {
  std::cerr << "Opening file:" << fn << " for write." << std::endl;
  std::fstream f(fn, std::fstream::out);
  if (!f.good()) {
    return false;
  }

  // Read # of rows and cols
  f << nr_row << " " << nr_col << " ";

  float data;
  std::cerr << "Matrix dimension: " << nr_row << "x" << nr_col << std::endl;
  for (int i = 0; i < v.size(); ++i) {
    f << v[i] << ' ';
  }
  f << "\n";
  return true;
}
